// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//

import clock from "clock";
import { HeartRateSensor } from "heart-rate";
import { today } from 'user-activity';
import { charger, battery } from "power";
import { logger, screens, sh_screen } from "../common/utils";
import * as utils from "../common/utils";
import { tz_data } from "../common/tz";
import * as names from "../common/names";
import * as C from "../common/constants";
import { P } from "../common/prefs";
import * as cfg1 from "../common/config1";
import * as timer from "../common/timer";

let screen;
let scr;
let scr_width;
let scr_height;
let moon_day = -1;

var heart_rate = 0;

var bpm_showing = true;
let hrm;
const days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]

export function init(doc)
{

  let bar;
  screen = doc.getElementById("analog_face");
  scr = utils.scr_init(screen, null);
  adj_width(doc.getElementById("root"));

  utils.image_btn(screen, "lr_btn", "icons/moon-0.png", (btn) => {
    if (timer.running)
      timer.set_lap();
    else {
      cfg1.sh_screen();
    }
  });

  for (let i = 0; i < 12; i++) {
    bar = screen.getElementById("tick-" + i);
    bar.groupTransform.rotate.angle = (i * 30);
  }

  let stats_click = screen.getElementById("stats_click");
  stats_click.addEventListener("click", (evt) => {
    bpm_showing = !bpm_showing;
    screen.getElementById("stats_img").image = (bpm_showing ? "icons/bpm.png" :
                                                              "icons/steps.png");
    sh_stats();
  })

  let time2_click = screen.getElementById("time2_click");
  time2_click.addEventListener("click", (evt) => {
    timer.sh_screen();
  })
  timer_show(false);

  tz_data[C.TZ_CUST1_IDX].off = P.custom_tz1;
  tz_data[C.TZ_CUST2_IDX].off = P.custom_tz2;

  hrm = new HeartRateSensor();
  hrm.onreading = function() {
    if (hrm.heartRate)
      heart_rate = hrm.heartRate;
    hrm.stop();
  }

  set_rotation();
  set_moon("icons/moon-0.png");
  return 0;
}

export function sh_screen()
{

  timer_show(timer.enable);
}

//
//  The Ionic is the only Fitbit device with a non-square
//  screen (it's wider than it is tall).  Since all of the UI
//  elements are proportional to the height of the screen
//  this means that elements in the middle row of the
//  face (the '9', day of week, day of month and '3') would
//  be wrong on the Ionic.  This routine adjusts the elements
//  properly (do the adjustment for all devices, it winds up
//  being a no-op for square screens).
//
function adj_width(r)
{
  var e;

  scr_width = r.width;
  scr_height = r.height;
//logger("Screen: " + r.width + "x" + r.height);
  if (r.width == r.height)
    return;

  let c1 = r.width / 2;
  let c2 = r.height / 2;
  e = screen.getElementById("min-3");
  e.x = (c1 + c2) - 42;
  e = screen.getElementById("date_box");
  e.x = (c1 + c2) - 96;
  e = screen.getElementById("min-9");
  e.x = (c1 - c2) + 42;
  e = screen.getElementById("week");
  e.x = (c1 - c2) + 50;
}

export function timer_show(enable)
{

  if (enable) {
    screen.getElementById("time2").style.display = "inline";
    screen.getElementById("time2_icon").style.display = "none";
  } else {
    screen.getElementById("time2").style.display = (P.tz2 == C.TZ_NONE2) ? "none" : "inline";
    screen.getElementById("time2_icon").style.display = (P.tz2 == C.TZ_NONE2) ? "inline" : "none";
  }
}

export function set_rotation()
{

  let min3 = screen.getElementById("min-3");
  let min9 = screen.getElementById("min-9");
  if (P.rotation < 0) {
    min3.text = "9";
    min9.text = "3";
  } else {
    min3.text = "3";
    min9.text = "9";
  }
}

function moon_image(local)
{

  if (local == null)
    local = new Date();
  moon_day = local.getDate();
  return utils.moon_phase(local);
}

export function set_moon(image)
{

  if (image == null)
    image = moon_image(null);
  let btn = screen.getElementById("lr_btn");
  let lr_icon = btn.getElementById("b_image");
  lr_icon.image = image;
}

export function sh_moon(local)
{

  if (P.moon && (local.getDate() != moon_day))
    set_moon(moon_image(local));
}

// Returns an angle (0-360) for the current hour in the day, including minutes
function hoursToAngle(hours, minutes) {
  let hourAngle = (360 / 12) * hours;
  let minAngle = (360 / 12 / 60) * minutes;
  return (hourAngle + minAngle) * P.rotation;
}

// Returns an angle (0-360) for minutes
function minutesToAngle(minutes) {
  return ((360 / 60) * minutes) * P.rotation;
}

// Returns an angle (0-360) for seconds
function secondsToAngle(seconds) {
  return ((360 / 60) * seconds) * P.rotation;
}

function sh_battery() {
  
  let lvl = Math.floor(battery.chargeLevel);
  let batValue = screen.getElementById("battery");
  batValue.text = lvl;
  if (lvl > 50)
    batValue.style.fill = "lime";
  else if (lvl > 25)
    batValue.style.fill = "yellow";
  else
    batValue.style.fill = "red";
}

function get_time2(tz2_idx, utc_time, tm2)
{
  var tz, time;
  
  if (timer.enable) {
    return timer.state();
  } else if (tz2_idx >= 0) {
    tz = utils.find_tz(tz2_idx, utc_time);
    let hr = tm2.getHours();
    let ampm = utils.ampm_mark(hr, P, " A", " P");
    if (!P.hr24 && (hr > 12))
      hr -= 12;
    let mn = tm2.getMinutes();
    time = utils.numPad(hr, " ") + ":" + utils.numPad(mn, "0");
    if (!P.hr24)
      time += ampm;
  } else {
    tz = "";
    time = "";
  }
  return { tz, time };
}

function sh_tz2(tz2_idx, tm2, utc_time)
{

  let { tz, time } = get_time2(tz2_idx, utc_time, tm2);
  let time2 = screen.getElementById("time2");
  time2.getElementById("tz2").text = tz;
  utils.text_bck(time, time2);
}

function sh_stats()
{

  let txt = bpm_showing ? heart_rate : utils.comma(today.adjusted.steps);
  utils.text_bck(txt, screen.getElementById("stats"));
}

// Rotate the hands every tick
export function updateFace()
{

  let local = new Date();
  let tm1 = local;
  let tm2;
  let tz1_idx = names.get_info(P.tz1).tz;
  let tz2_idx = names.get_info(P.tz2).tz;
  if (tz1_idx > 0 || tz2_idx >= 0) {
    let off = utils.utc_off(tm1.toString());
    let utc_time = utils.utc(tm1, -off);
    if (tz1_idx > 0)
      tm1 = utils.utc(utc_time, utils.tz_off(tz1_idx, utc_time));
    if (tz2_idx > 0)
      tm2 = utils.utc(utc_time, utils.tz_off(tz2_idx, utc_time));
    else if (tz2_idx == 0)
      tm2 = local;
  }
  let hours = tm1.getHours();
  let mins = tm1.getMinutes();
  let secs = tm1.getSeconds();

  screen.getElementById("pm").text = utils.ampm_mark(hours, P, "AM", "PM");

  let hourHand = screen.getElementById("hours");
  hourHand.groupTransform.rotate.angle = hoursToAngle(hours%12, mins);

  let minHand = screen.getElementById("mins");
  minHand.groupTransform.rotate.angle = minutesToAngle(mins);

  let secHand = screen.getElementById("secs");
  secHand.groupTransform.rotate.angle = secondsToAngle(secs);

  screen.getElementById("tz1").text = ((tz1_idx == 0) ? "" : utils.find_tz(tz1_idx, utc_time));
  screen.getElementById("date").text = tm1.getDate();
  screen.getElementById("week").text = days[tm1.getDay()];
  
  // upper left corner
  sh_battery();
  
  // upper right corner
  hrm.start();
  sh_stats();

  // lower left corner
  sh_tz2(tz2_idx, tm2, utc_time);

  // lower right corner
  sh_moon(local);
}

export function updateClock()
{

  if (!timer.running)
    updateFace();
}
