// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import * as C from "../common/constants";
import { logger } from "../common/utils";
import * as utils from "../common/utils";
import { P } from "../common/prefs";
import * as prefs from "../common/prefs";
import * as names from "../common/names";
import * as cfg_select from "../common/cfg_select";

let scr;
var table_no;

var hi;
var max;
var lineno = 0;
var line = [];

export function init(doc)
{
  var btn;

  let screen = doc.getElementById("cfg_time");
  scr = utils.scr_init(screen, "");

  utils.text_btn(screen, "u_btn", "Select City", "fb-white", "fb-blue", (btn) => {
    cfg_select.sh_screen();
  });

  utils.image_btn(screen, "ul_btn", "icons/up.png", (btn) => {
    if (lineno >= 5)
      lineno -= 5;
    sh_page();
  });

  utils.image_btn(screen, "ur_btn", "icons/down.png", (btn) => {
    if (lineno < (max - 5))
      lineno += 5;
    sh_page();
  });

  for (let i = 0; i < 5; i++) {
    line[i] = { name: screen.getElementById("name" + i), bg: screen.getElementById("name" + i + "_bg")};
    btn = screen.getElementById("name" + i + "_touch");
    btn.addEventListener("click", (evt) => {
      let idx = tz_index(i + lineno);
      let info = names.get_info(idx);
      if (info.tz != C.TZ_HEADER) {
        if (table_no == C.TZ_TILE1)
          P.tz1 = idx;
        else
          P.tz2 = idx;
        prefs.save();
        hi = idx;
        sh_page();
      }
    })
  }

  return scr;
}

export function sh_screen(no)
{
  let pre;

  table_no = no;
  pre = 2;
  hi = (no == C.TZ_TILE1) ? P.tz1 : P.tz2;
  max = pre + names.num_cities() + names.num_countries();
  lineno = (hi < 0) ? 0 : (parseInt((pre + hi)/5) * 5);
  sh_page();
  utils.sh_screen(scr);
}

function tz_index(idx)
{

  let spec = 2;
  if (idx >= spec)
    return idx - spec;
  switch (idx) {

  case 0:
    return table_no == C.TZ_TILE1 ? C.TZ_LOCAL1 : C.TZ_NONE2;

  case 1:
    return table_no == C.TZ_TILE1 ? C.TZ_CUSTOM1 : C.TZ_CUSTOM2;

  case 2:
    return C.TZ_TIMER;

  }
  return 0;
}

function get_bg(idx, info)
{

  if (info.tz == -2)
    return "blue";
  return (idx == hi) ? "grey" : "black";
}

function sh_page()
{
  var info;
  var idx;

  for (let i = 0; i < 5; i++) {
    idx = tz_index(i + lineno);
    info = names.get_info(idx);
    if (info) {
      line[i].name.text = info.name;
      line[i].bg.style.fill = get_bg(idx, info);
    } else {
      line[i].name.text = "";
      line[i].bg.style.fill = "black";
    }
  }
}
