// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import * as C from "../common/constants";
import { logger } from "../common/utils";
import * as utils from "../common/utils";
import * as prefs from "../common/prefs";
import { P } from "../common/prefs";
import { tz_data } from "../common/tz"
import * as face from "../common/analog_face";
import * as cfg_select from "../common/cfg_select";
import * as config2 from "../common/config2";

export let scr;

let tz_scr;
export let tz_idx;

export function init(doc)
{

  let screen = doc.getElementById("config1");
  scr = utils.scr_init(screen, "Config", null);
  utils.image_btn(screen, "ul_btn", "icons/left_btn.png", (btn) => {
    config2.sh_screen();
  });
  utils.image_btn(screen, "ur_btn", "icons/right_btn.png", (btn) => {
    cfg_select.sh_screen();
  });

  //
  // AM/PM display
  //
  //  cfg_ampm - show/hide am/pm indicator
  //  cfg_am   - show am & pm indicators
  //  cfg_pm   - show pm only
  //
  //  P.ampm bit flag controlling indicators
  //    0/1 = PM only, AM/PM
  //    0/2 = show AM/PM at all
  //
  let cfg_ampm = screen.getElementById("cfg_ampm");
  cfg_ampm.image = (P.ampm & 1) ? "icons/on.png" : "icons/off.png";
  cfg_ampm.addEventListener("click", (evt) => {
    P.ampm ^= 1;
    cfg_ampm.image = (P.ampm & 1) ? "icons/on.png" : "icons/off.png";
    prefs.save();
  })
  let cfg_text = screen.getElementById("cfg_am_text");
  cfg_text.text = (P.ampm & 2) ? "PM" : "AM/PM";  
  let cfg_am = screen.getElementById("cfg_am");
  cfg_am.addEventListener("click", (evt) => {
    P.ampm ^= 2;
    cfg_text.text = (P.ampm & 2) ? "PM" : "AM/PM";  
    prefs.save();
  })

  //
  // 12/24 hour display
  //
  let cfg_hr12_bck = screen.getElementById("cfg_hr12_bck");
  let cfg_hr24_bck = screen.getElementById("cfg_hr24_bck");
  cfg_hr12_bck.style.fill = P.hr24 ? "black" : "red";
  cfg_hr24_bck.style.fill = P.hr24 ? "red" : "black";

  let cfg_hr12 = screen.getElementById("cfg_hr12");
  cfg_hr12.addEventListener("click", (evt) => {
    cfg_hr12_bck.style.fill = "red";
    cfg_hr24_bck.style.fill = "black";
    P.hr24 = 0;
    prefs.save();
  })
  let cfg_hr24 = screen.getElementById("cfg_hr24");
  cfg_hr24.addEventListener("click", (evt) => {
    cfg_hr24_bck.style.fill = "red";
    cfg_hr12_bck.style.fill = "black";
    P.hr24 = 1;
    prefs.save();
  })

  //
  // rotation direction
  //
  let cfg_rotate = screen.getElementById("cfg_rotate");
  cfg_rotate.image = (P.rotation > 0) ? "icons/cw.png" : "icons/ccw.png";
  cfg_rotate.addEventListener("click", (evt) => {
    P.rotation = (P.rotation > 0) ? -1 : 1;
    cfg_rotate.image = (P.rotation > 0) ? "icons/cw.png" : "icons/ccw.png";
    face.set_rotation();
    prefs.save();
  })
  
  return scr;
}

export function sh_screen()
{

  utils.sh_screen(scr);
}
