// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import { vibration } from "haptics";
import * as C from "../common/constants";
import { logger } from "../common/utils";
import * as utils from "../common/utils";
import * as prefs from "../common/prefs";
import { P } from "../common/prefs";
import { tz_data } from "../common/tz"
import * as face from "../common/analog_face";
import * as get_time from "../common/get_time";
import * as laps from "../common/laps";

let screen;
export let scr;
let ur_btn;

export var enable = false;

var mode = C.TIMER_UP;

let timer_value;

export var running;
var st_time;
var end_time;
var init_time = 0;

//
// timer interrupts
//
var fire = 0;
var haptic = 0;

export function init(doc)
{
  var btn, txt, img;

  screen = doc.getElementById("timer");
  scr = utils.scr_init(screen, "Timer");

  utils.text_btn(screen, "u_btn", "Chronograph", "fb-white", "fb-blue", (btn) => {
    utils.sh_screen(0);
  });

  utils.image_btn(screen, "ur_btn", "icons/right_btn.png", (btn) => {
    laps.sh_screen();
  });
  ur_btn = screen.getElementById("ur_btn");

  timer_value = screen.getElementById("timer_value");
  timer_value.addEventListener("click", (evt) => {
    if (mode == C.TIMER_DOWN)
      get_time.sh_screen(scr, 6, init_time / 1000, save);
  })

  btn = screen.getElementById("timer_enable");
  let e_txt = btn.getElementById("b_text");
  utils.text_btn(screen, "timer_enable", "Disabled", "fb-yellow", "fb-red", (btn) => {
    enable = !enable;
    e_txt.text = (enable ? "Enabled" : "Disabled");
    face.timer_show(enable);
  })

  utils.text_btn(screen, "timer_clear", "Reset", "fb-yellow", "fb-red", (btn) => {
    st_time = Date.now();
    end_time = st_time;
    laps.reset();
    let ticks = (mode == C.TIMER_UP) ? (end_time - st_time) : init_time;
    timer_value.text = utils.milli2str(ticks, true);
  })
  
  btn = screen.getElementById("timer_mode");
  let m_txt = btn.getElementById("b_text");
  utils.text_btn(screen, "timer_mode", "Up", "fb-yellow", "fb-red", (btn) => {
    if (mode == C.TIMER_DOWN) {
      mode = C.TIMER_UP
      end_time = st_time;
      m_txt.text = "Up";
      ur_btn.style.display = "inline";
    } else {
      mode = C.TIMER_DOWN;
      end_time = st_time + init_time;
      m_txt.text = "Down";
      ur_btn.style.display = "none";
    }
    timer_value.text = utils.milli2str(end_time - st_time, true);
    laps.reset();
  })

  st_time = Date.now();
  end_time = st_time;
  laps.reset();
  
  return scr;
}

export function sh_screen()
{

  timer_value.text = utils.milli2str(elapsed(Date.now()), true);
  utils.sh_screen(scr);
}

function save(offset)
{

  init_time = offset * 1000;
  end_time = st_time;
  timer_value.text = utils.milli2str(init_time, true);
}

export function start()
{

  laps.elapsed(0);
  var ticks = end_time - st_time;
  running = true;
  st_time = Date.now() - ticks;
  if (mode == C.TIMER_UP)
    face.set_moon("icons/lap.png");
  sched_intr(1000, ticks);
}

export function stop()
{

  end_time = Date.now();
  laps.elapsed(end_time - st_time);
  running = false;
  timer_value.text = utils.milli2str(end_time - st_time, true);
  face.set_moon(null);
  let ticks = end_time - st_time;
  sched_intr(1000, ticks);
}

export function start_stop()
{

  if (running)
    stop();
  else
    start();
}

export function set_lap()
{

  laps.set(Date.now() - st_time);
  laps.elapsed(0);
}

export function elapsed(ticks)
{

  if (running)
    ticks -= st_time;
  else
    ticks = end_time - st_time;
  return (mode == C.TIMER_UP) ? ticks : init_time - ticks;
}

function haptic_intr()
{

  vibration.stop();
  haptic = 0;
}

function do_haptic(state, time)
{


  time = parseInt(time / 1000);
  if (state && ((time % 10) == 0)) {
    let type = "alert";
    vibration.start(type);
    haptic = setTimeout(haptic_intr, 2000);
  }
}

export function state()
{
  let tz;

  let e = elapsed(Date.now());
  if (running) {
    if (mode == C.TIMER_UP) {
      tz = "lap " + (laps.lapno + 1);
    } else {
      if (e <= 0) {
        tz = "elapsed";
        do_haptic(running, e);
      } else {
        tz = "down";
      }
    }
  } else if (mode == C.TIMER_DOWN)
    tz = (e == init_time) ? "timer" : "stopped";
  else
    tz = (e == 0) ? "chrono" : "pause";
  let time = utils.milli2str(e, false);
  return { tz, time };
}

function timer_intr()
{

  face.updateClock();
  sched_intr(1000, Date.now() - st_time);
}

export function sched_intr(delay, ticks)
{

  if (fire != 0) {
    clearTimeout(fire);
    fire = 0;
  }
  face.updateFace();
  if (running) {
    let t = parseInt(ticks / 1000);
    ticks = ticks - (t * 1000);
    fire = setTimeout(timer_intr, delay - ticks);
  }
}
