// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import { tz_data } from "../common/tz.js";
import * as C from "../common/constants.js";
import * as names from "../common/names";
import { P } from "../common/prefs";
import * as prefs from "../common/prefs";
import * as face from "../common/analog_face";

let log_enable = 1;

export let screens = [];

export function logger(str)
{

  if (log_enable)
    console.log("DDD: " + str);
}

export function pr_obj(title, object, details) {

  logger(title);
  for(let key in object) {
    if (details) {
      try {
       logger('  ' + key + ':' + JSON.stringify(object[key]));
      } catch (error) {
        // Some values throw an error when trying to access them.
        logger('  ' + key + '|' + error.message);
     }
    } else {
      logger('  ' + key + "-");
    }
  }
}

// Add zero in front of numbers < 10
export function numPad(i, p) {
  if (i < 10) {
    i = p + i;
  }
  return i;
}

export function sw_screen(s1, s2)
{
  s1.style.display = "inline";
  s2.style.display = "none";
}

export function init_face(doc)
{
  let bar;

  for (let i = 0; i < 12; i++) {
    bar = doc.getElementById("tick-" + i);
    bar.groupTransform.rotate.angle = (i * 30);
  }

  screens[0] = doc.getElementById("analog_face");
}

export function image_btn(screen, id, image, action)
{

  let btn = screen.getElementById(id);
  if (btn) {
    let img = btn.getElementById("b_image");
    img.image = image;
    let touch = btn.getElementById("b_touch");
    touch.addEventListener("click", (evt) => {
      action(btn);
    })
  }
}

export function text_btn(screen, id, title, c_fore, c_back, action)
{

  let btn = screen.getElementById(id);
  if (btn) {
    let back = btn.getElementById("b_back");
    back.style.fill = c_back;
    let fore = btn.getElementById("b_text");
    fore.style.fill = c_fore;
    fore.text = title;
    let touch = btn.getElementById("b_touch");
    touch.addEventListener("click", (evt) => {
      action(btn);
    })
  }
}

export function scr_init(screen, title)
{

  let scr = screens.length;
  screens[scr] = screen;

  text_btn(screen, "um_btn", title, "fb-white", "fb-blue", (btn) => {
    sh_screen(0);
  });
  return scr;
}

export function sh_screen(idx)
{

  let max = screens.length;
  for (let i = 0; i < max; i++)
    screens[i].style.display = "none";
  screens[idx].style.display = "inline";
  if (idx == 0)
    face.sh_screen();
}

export function utc(time, off)
{

  let yr = time.getFullYear();
  let mo = time.getMonth();
  let day = time.getDate();
  let hr = time.getHours();
  let mn = time.getMinutes();
  let sec = time.getSeconds();

  let d = 0;
  let min = ((hr * 60) + mn) + off;
  if (min < 0) {
    d = -1;
    min += (24 * 60);
  } else if (min >= (24 * 60)) {
    d = 1;
    min -= (24 * 60);
  }
  hr = parseInt(min / 60);
  min -= (hr * 60);
  return new Date(yr, mo, day, hr, min, sec);
}

export function utc_off(gmt)
{
  let sign;

  gmt = gmt.substring(gmt.indexOf("GMT") + 3);
  let sign = gmt.charAt(0);
  let idx = gmt.indexOf(":");
  let h = parseInt(gmt.substring(1, idx));
  let m = parseInt(gmt.substring(idx + 1));
  let r = (h * 60) + (m * 1);
  if (sign == "-")
    r = -r;
  return r;
}

const mo_days = [  31,  28,  31,  30,  31,  30,  31,  31,  30,  31,  30,  31 ];
const mo_cumm = [   0,  31,  59,  90, 120, 151, 181, 212, 243, 273, 304, 334, 365 ];
function find_dom(day, month, year, begin)
{

  let time = new Date(year, month);   // month is 1 based
  let first = time.getDay();          // 0 based
  let off = day - first;
  if (off < 0)
    off += 7;
  let dom = 1 + off;
  if (begin)
    return dom;
  while ((dom + 7) <= mo_days[month])
    dom += 7;
  return dom;
}

//  Starts at 2AM on the 2nd Sunday in March
//  Ends at 2AM on the 1st Sunday in November
//
function dst_usa(time)
{

  let month = time.getMonth();
  if (month < 2 || month > 10)
    return false;
  if (month != 2 && month != 10)
    return true;

  let first = find_dom(0, month, time.getFullYear(), true);
  let off = (month == 2) ? 7 : 0;
  let anchor = new Date(time.getFullYear(), month, first + off, 2, 0, 0);
  if (month == 2) {
    return (time.getTime() < anchor.getTime()) ? false : true;
  } else {
    return (time.getTime() < anchor.getTime()) ? true : false;
  }
}

export function dig2(num, lead)
{

  return (num < 10) ? (lead + num) : num;
}

export function milli2str(milli, hundred)
{
  let sign;

  if (milli < 0) {
    sign = "-";
    milli = -milli;
  } else
    sign = "";
  milli = parseInt(milli / 10);
  let sec = parseInt(milli/100);
  let hun = milli - (sec * 100);
  let min = parseInt(sec / 60);
  sec = sec - (min * 60);
  let hr = parseInt(min / 60);
  min -= hr * 60;
  let ret = "";
  if (hundred) {
    ret = "." + numPad(hun, "0");
    if (hr == 0 && min == 0)
      return sign + sec + ret;
  }
  ret = numPad(sec, "0") + ret;
  if (hr == 0)
    return sign + min + ":" + ret;
  return sign + hr + ":" + numPad(min, "0") + ":" + ret;
}

//  Starts at 1AM UTC on the last Sunday in March
//  Ends at 1AM UTC on the last Sunday in October
//
function dst_eu(time)
{

  let month = time.getMonth();
  if (month < 2 || month > 9)
    return false;
  if (month != 2 && month != 9)
    return true;

  let last = find_dom(0, month, time.getFullYear(), false);
  let tstr = time.getFullYear() + "-" + dig2(month + 1, "0") + "-" + dig2(last, "0") + "T01:00:00Z";
  let anchor = new Date(tstr);
  if (month == 2) {
    return (time.getTime() < anchor.getTime()) ? false : true;
  } else {
    return (time.getTime() < anchor.getTime()) ? true : false;
  }
}

function dst(idx, time)
{
  let mo_day;

  if (tz_data[idx].days == 0)
    return dst_usa(time);
  if (tz_data[idx].days == 1)
    return dst_eu(time);
  return false;
}

export function find_tz(idx, utc)
{

  if (dst(idx, utc))
    return tz_data[idx].daylight;
  return tz_data[idx].standard;
}

export function tz_off(idx, utc)
{

  let off = tz_data[idx].off;
  if (dst(idx, utc))
    off += 60;
  return off;
}

export function ampm_mark(h, p, am_str, pm_str)
{

  if (p.hr24)
    return "";
  switch (p.ampm) {

  case 0:   // cfg_ampm = 0, cfg_am = 0
  case 2:   // cfg_ampm = 0, cfg_am = 1
    return "";

  case 3:   // cfg_ampm = 1, cfg_am = 0
    return (h >= 12) ? pm_str : "";

  case 1:   // cfg_ampm = 1, cfg_am = 1
    return (h >= 12) ? pm_str : am_str;


  }
  return "?";
}

export function get_ampm(w)
{

  let btn = w.getElementById("cfg_ampm0");
  if (btn.value != 0)
    return 0;
  btn = w.getElementById("cfg_ampm1");
  if (btn.value != 0)
    return 1;
  btn = w.getElementById("cfg_ampm2");
  if (btn.value != 0)
    return 2;
}

export function epoch_days(d)
{

  let yr = d.getFullYear();
  if (d.getMonth() < 2)
    yr = yr - 1;
  let leaps = parseInt(yr/4) - parseInt(yr/100) + parseInt(yr/400);

  let days = (d.getFullYear() * 365) + d.getDate() + mo_cumm[d.getMonth()];
  return days + leaps;
}

export function moon_day(d)
{

  return parseInt((d + 1) % 29.530588853);
}

export function moon_icon(mday)
{

  mday = mday + 2;
  let d = moon_day(mday);
  let cycle = (moon_day(mday + 29) == d) ? 29 : 30;
  return parseInt((d * 8) / cycle);
}

export function lunar_day(d)
{

  let ndays = epoch_days(d) - epoch_days(new Date(1900, 0, 1, 0, 0, 0));
  return moon_day(ndays);
}

export function moon_phase(d)
{

  let ndays = epoch_days(d) - epoch_days(new Date(1900, 0, 1, 0, 0, 0));
  return "icons/moon-" + moon_icon(ndays) + ".png";
}

export function tz_split(name)
{

  let str = name.split(";");
  return { name: str[0], tz: parseInt(str[1]) };
}

export function comma(n)
{

  if (n < 1000)
    return "" + n;
  let u = parseInt(n / 1000);
  n = n - (u * 1000);
  var sep = ",";
  if (n < 100)
    sep += "0";
  if (n < 10)
    sep += "0";
  return u + sep + n;
}

//  Text with background
//
//  txt - text to display
//  t - text UI element for txt
//  b - background rectangle for the text
//  r - x coord of right side of text (optional)
//
//  Stretch the background box to cover the width of the
//  text.  If r is specified then the text is right
//  justified and r is the x coord of the right hand side.
//
export function text_bck(txt, e)
{

  let t = e.getElementById("info");
  t.text = txt;
  let bbox = t.getBBox();

  let b = e.getElementById("back");
  b.x = bbox.x;
  b.width = bbox.width;
}

export function mk_ascii(a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16)
{

  var r = "";
  let l = [ a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16 ];
  for (let i = 0; i < 16; i++) {
    if (!l[i])
      return r;
    r = r + l[i];
  }
  return r;
}
